;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; UC-X TV Demo
;; (c) Copyright 2007 Burnt Wasp and Figment Games
;;     All Rights Reserved.
;;
;; Filename:    uc_tv_demo.asm
;; Author:      Gary Preston
;; Created:     22/04/2007
;; Purpose:
;;   Demonstrate usage of uc_tv_driver
;;   Comments are a little more verbose than usual for this reason
;;
;; NOTES: 
;;   Requires ucasm, to assemble use:-
;;      ucasm.exe -N prop_tv.asm
;;
;;   To Upload use:
;;      UCXLoadder.exe -P prop_tv.binary
;;   
;;   Optionally use -Pe to program EEPROM
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                include "p8x32a.inc"
        
                clkmode	XTAL1_PLL16X
                clkfreq	96_000_000
                      
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Constants
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
fntsc           equ 3_579_545     ;NTSC color frequency
lntsc           equ 3640          ;NTSC color cycles per line * 16
sntsc           equ 624           ;NTSC color cycles per sync * 16

fpal            equ 4_433_618     ;PAL color frequency
lpal            equ 4540          ;PAL color cycles per line * 16
spal            equ 848           ;PAL color cycles per sync * 16

paramcount      equ 14
colortable      equ $180          ; start of colortable inside tv driver cog
display_base    equ $5000         ; start of tile data (16x12 tiles)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Initialisation cog
;    Starts TV Driver in separate cog passing in address of tv param block
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                cog  $020                   
 
InitEntry
                mov DIRA, LED_PIN
                cognew TVDriverEntry, @tv_status
  
:Loopy       
                ; debug LED - lights when tv driver not disabled (tv_status > 0)
                rdlong tv_status, tv_StatusAddr
                cmp tv_Status, #$01     wc
                
        if_nc   mov OUTA, LED_PIN       
        
                jmp     #:Loopy
 
 
                fit


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; TV Driver
;    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                ; Uses 1 cog - see asm file for config details
                include "uc_tv_driver_010.asm"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; DATA
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                
;               
; TV Driver Params copied over to param buffer
; See end of file for detailed param notes
;
tv_statusAddr           long     @tv_status     ; 
tv_status               long     0              ; tv_status 0/1/2 off/invis/visible read-only           
                        long     1              ; tv_enable 0/1
                        long     %001_0101      ; %pppmmmm  pin group, pin group mode
                        long     %01001         ; %tccip tile, chroma, interlace, ntsc/pal
                        long     @screen_ptrs   ; ptr to screen (words)
                        long     @color_palette ; ptr to colours (longs)
                        long     16             ; horizontal tiles
                        long     12             ; vertical tiles
                        long     10             ; horizontal tile expansion
                        long     1              ; vertical tile expansion
                        long     0              ; horizontal offset
                        long     0              ; vertical offset
                        long     0              ; broadcast frequency
                        long     0              ; aural fm cog

; Pin for Status/debug LED 
LED_PIN                 long    $00010000


; -----------------------------------------------------------------------------
; TV Screen Setup
; -----------------------------------------------------------------------------
; Setup a 16x12 tiled display comprised of 16x16 pixel tiles
; For demo purposes all 192 tile pointers use the same tile data
;
; screen_ptrs
;    tv_ht * tv_vt words which define screen contents.
;    Each word holds a 6bit colour set index and 10 bit ptr to actual tile data
;
;    %cccccc_pppppppppp   c = colour set index, p = tile data address
;
; Encoding TileData Ptr:
;    Tile Data is located in hub ram. The upper 10 bits of the 16bit address are stored in the
;    lower 10 bits of the word. As we're losing 6 bits of precision this means tile data must 
;    be located on a 64 byte boundary.
;    
; Encoding Colour Set index:
;    Colour Set index is simply a 6 bit value located in the upper 6 bits of the
;    screen ptr word which is used to index one of 64 possible colour sets in the 
;    colour table.
;

; Set all 16x12 tiles to use tile1 since that's all we have defined ;) 
screen_ptrs
i = 0
        while i < 192     
                word  (@tile1 >> 6) | ((i % 64) << 10) 
i = i + 1       
        end
     
; -----------------------------------------------------------------------------
; Colour Palette
; -----------------------------------------------------------------------------
;
; Colour Table:
;    The colour table contains 64 long entries, each entry made up of 4x8bit colours
;    This table will be copied into COG ram at the end of the tv driver code
;
;    bits 7..4 - chroma data
;    bit  3 - chroma on/off
;    bit  2..0 - luminance
;       Do not use liminance values of 0 or 1, these are used for signal SYNC
;       Valid values are 2..7
;    
; NOTE: chroma data is only enough for 16 unique colours (at least in 4 colour mode)
;       page 329 - prop manual mentions a 2 colour mode which splits the 32 bit colour 
;       parameter into 2 words rather than 4 bytes. Only mentions encoding for VGA though.
;
;       Colour_palette is read into cog ram at runtime during tv driver initialisation
;       to the location specified by "colortable"
;
color_palette           long    ($00001010 * ((0+4) & $F)) + $2B060C02 
i = 1
  while i < 64
  long    ($00001010 * ((i+4) & $F)) + $2B060C02 
i = i + 1
  end


; -----------------------------------------------------------------------------
; Tile Data                                
; -----------------------------------------------------------------------------
;
; Tile Data
;    With 16x16 pixel tiles, the tiledata requires tileheight longs, E.G 16 longs
;    Each long contains 2 bits per pixel thus 16 four colour pixels
;    where the colour is an index into the colour set specified in the pixelGroupPtr
;
; 192 tiles would provide one unique tile for each of the 16x12 tiles making up
; the demo screen with run time modification of the tile data then providing 
; a framebuffer for graphics routines
;
; This demo has just a single tile which the entire screen uses

                        ; Tile data should begin on a 64byte boundary
                        ; NOTE Last byte of RAM = $7FFF but image must finish 8 bytes earlier
                        hub display_base
                        
TileData

; UC
tile1       long %0000000000000000_0000000000000000   
            long %0000000000000000_0000000000000000
            long %0000010101010101_0001010000000101
            long %0000010101010101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000000000000101_0001010000000101
            long %0000010101010101_0001010101010101
            long %0000010101010101_0001010101010101
